package java.lang;

import sun.misc.VM;

public final class Integer extends Number implements Comparable<Integer>
{
    public static final int MIN_VALUE = -2147483648;
    public static final int MAX_VALUE = 2147483647;
    public static final Class<Integer> TYPE;
    static final char[] digits;
    static final char[] DigitTens;
    static final char[] DigitOnes;
    static final int[] sizeTable;
    private final int value;
    public static final int SIZE = 32;
    public static final int BYTES = 4;
    private static final long serialVersionUID = 1360826667806852920L;
    
    public static String toString(int i, int n) {
        if (n < 2 || n > 36) {
            n = 10;
        }
        if (n == 10) {
            return toString(i);
        }
        char[] array = new char[33];
        int n2 = i < 0;
        int n3 = 32;
        if (n2 == 0) {
            i = -i;
        }
        while (i <= -n) {
            array[n3--] = Integer.digits[-(i % n)];
            i /= n;
        }
        array[n3] = Integer.digits[-i];
        if (!(n2 == 0)) {
            array[--n3] = '-';
        }
        return new String(array, n3, 33 - n3);
    }
    
    public static String toUnsignedString(int n, int n2) {
        return Long.toUnsignedString(toUnsignedLong(n), n2);
    }
    
    public static String toHexString(int n) {
        return toUnsignedString0(n, 4);
    }
    
    public static String toOctalString(int n) {
        return toUnsignedString0(n, 3);
    }
    
    public static String toBinaryString(int n) {
        return toUnsignedString0(n, 1);
    }
    
    private static String toUnsignedString0(int n, int n2) {
        int max = Math.max((32 - numberOfLeadingZeros(n) + (n2 - 1)) / n2, 1);
        char[] array = new char[max];
        formatUnsignedInt(n, n2, array, 0, max);
        return new String(array, true);
    }
    
    static int formatUnsignedInt(int n, int n2, char[] array, int n3, int n4) {
        int n5 = (1 << n2) - 1;
        do {
            array[n3 + --n4] = Integer.digits[n & n5];
            n >>>= n2;
        } while (n != 0 && n4 > 0);
        return n4;
    }
    
    public static String toString(int n) {
        if (n == Integer.MIN_VALUE) {
            return "-2147483648";
        }
        int n2 = (n < 0) ? (stringSize(-n) + 1) : stringSize(n);
        char[] array = new char[n2];
        getChars(n, n2, array);
        return new String(array, true);
    }
    
    public static String toUnsignedString(int n) {
        return Long.toString(toUnsignedLong(n));
    }
    
    static void getChars(int i, int n, char[] array) {
        int n2 = '\0';
        if (i < 0) {
            n2 = '-';
            i = -i;
        }
        while (i >= 65536) {
            int n3 = i / 100;
            int n4 = i - ((n3 << 6) + (n3 << 5) + (n3 << 2));
            i = n3;
            array[--n] = Integer.DigitOnes[n4];
            array[--n] = Integer.DigitTens[n4];
        }
        do {
            int n5 = i * 52429 >>> 19;
            array[--n] = Integer.digits[i - ((n5 << 3) + (n5 << 1))];
            i = n5;
        } while (i != 0);
        if (n2 != '\0') {
            array[--n] = n2;
        }
    }
    
    static int stringSize(int i) {
        int n;
        for (n = 0; i > Integer.sizeTable[n]; ++n) {}
        return n + 1;
    }
    
    public static int parseInt(String s, int n) throws NumberFormatException {
        if (s == null) {
            throw new NumberFormatException("null");
        }
        if (n < 2) {
            throw new NumberFormatException("radix " + n + " less than Character.MIN_RADIX");
        }
        if (n > 36) {
            throw new NumberFormatException("radix " + n + " greater than Character.MAX_RADIX");
        }
        int n2 = 0;
        boolean n3 = false;
        int i = 0;
        int length = s.length();
        int n4 = -2147483647;
        if (length > 0) {
            int char1 = s.charAt(0);
            if (char1 < '0') {
                if (char1 == '-') {
                    n3 = true;
                    n4 = Integer.MIN_VALUE;
                }
                else if (char1 != '+') {
                    throw NumberFormatException.forInputString(s);
                }
                if (length == 1) {
                    throw NumberFormatException.forInputString(s);
                }
                ++i;
            }
            int n5 = n4 / n;
            while (i < length) {
                int digit = Character.digit(s.charAt(i++), n);
                if (digit < 0) {
                    throw NumberFormatException.forInputString(s);
                }
                if (n2 < n5) {
                    throw NumberFormatException.forInputString(s);
                }
                int n6 = n2 * n;
                if (n6 < n4 + digit) {
                    throw NumberFormatException.forInputString(s);
                }
                n2 = n6 - digit;
            }
            return n3 ? n2 : (-n2);
        }
        else {
            throw NumberFormatException.forInputString(s);
        }
    }
    
    public static int parseInt(String s) throws NumberFormatException {
        return parseInt(s, 10);
    }
    
    public static int parseUnsignedInt(String s, int n) throws NumberFormatException {
        if (s == null) {
            throw new NumberFormatException("null");
        }
        int length = s.length();
        if (length > 0) {
            if (s.charAt(0) == '-') {
                throw new NumberFormatException(String.format("Illegal leading minus sign on unsigned string %s.", new Object[] { s }));
            }
            if (length <= 5 || (n == 10 && length <= 9)) {
                return parseInt(s, n);
            }
            long long1 = Long.parseLong(s, n);
            if ((long1 & 0xFFFFFFFF00000000L) == 0x0L) {
                return (int)long1;
            }
            throw new NumberFormatException(String.format("String value %s exceeds range of unsigned int.", new Object[] { s }));
        }
        else {
            throw NumberFormatException.forInputString(s);
        }
    }
    
    public static int parseUnsignedInt(String s) throws NumberFormatException {
        return parseUnsignedInt(s, 10);
    }
    
    public static Integer valueOf(String s, int n) throws NumberFormatException {
        return valueOf(parseInt(s, n));
    }
    
    public static Integer valueOf(String s) throws NumberFormatException {
        return valueOf(parseInt(s, 10));
    }
    
    public static Integer valueOf(int n) {
        if (n >= -128 && n <= Integer.IntegerCache.high) {
            return Integer.IntegerCache.cache[n + 128];
        }
        return new Integer(n);
    }
    
    public Integer(int value) {
        this.value = value;
    }
    
    public Integer(String s) throws NumberFormatException {
        this.value = parseInt(s, 10);
    }
    
    @Override
    public byte byteValue() {
        return (byte)this.value;
    }
    
    @Override
    public short shortValue() {
        return (short)this.value;
    }
    
    @Override
    public int intValue() {
        return this.value;
    }
    
    @Override
    public long longValue() {
        return (long)this.value;
    }
    
    @Override
    public float floatValue() {
        return (float)this.value;
    }
    
    @Override
    public double doubleValue() {
        return (double)this.value;
    }
    
    @Override
    public String toString() {
        return toString(this.value);
    }
    
    @Override
    public int hashCode() {
        return hashCode(this.value);
    }
    
    public static int hashCode(int n) {
        return n;
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof Integer && this.value == (int)o;
    }
    
    public static Integer getInteger(String s) {
        return getInteger(s, null);
    }
    
    public static Integer getInteger(String s, int n) {
        Integer integer = getInteger(s, null);
        return (integer == null) ? valueOf(n) : integer;
    }
    
    public static Integer getInteger(String s, Integer n) {
        String property = null;
        try {
            property = System.getProperty(s);
        }
        catch (IllegalArgumentException | NullPointerException ex) {}
        if (property != null) {
            try {
                return decode(property);
            }
            catch (NumberFormatException ex2) {}
            return n;
        }
        return n;
    }
    
    public static Integer decode(String s) throws NumberFormatException {
        int n = 10;
        int n2 = 0;
        boolean n3 = false;
        if (s.length() == 0) {
            throw new NumberFormatException("Zero length string");
        }
        int char1 = s.charAt(0);
        if (char1 == '-') {
            n3 = true;
            ++n2;
        }
        else if (char1 == '+') {
            ++n2;
        }
        if (s.startsWith("0x", n2) || s.startsWith("0X", n2)) {
            n2 += 2;
            n = 16;
        }
        else if (s.startsWith("#", n2)) {
            ++n2;
            n = 16;
        }
        else if (s.startsWith("0", n2) && s.length() > 1 + n2) {
            ++n2;
            n = 8;
        }
        if (s.startsWith("-", n2) || s.startsWith("+", n2)) {
            throw new NumberFormatException("Sign character in wrong position");
        }
        Integer value2;
        try {
            Integer value = valueOf(s.substring(n2), n);
            value2 = (n3 ? valueOf(-(int)value) : value);
        }
        catch (NumberFormatException ex) {
            value2 = valueOf(n3 ? ("-" + s.substring(n2)) : s.substring(n2), n);
        }
        return value2;
    }
    
    @Override
    public int compareTo(Integer n) {
        return compare(this.value, n.value);
    }
    
    public static int compare(int n, int n2) {
        return (n < n2) ? -1 : ((n == n2) ? 0 : 1);
    }
    
    public static int compareUnsigned(int n, int n2) {
        return compare(n + Integer.MIN_VALUE, n2 + Integer.MIN_VALUE);
    }
    
    public static long toUnsignedLong(int n) {
        return (long)n & 0xFFFFFFFFL;
    }
    
    public static int divideUnsigned(int n, int n2) {
        return (int)(toUnsignedLong(n) / toUnsignedLong(n2));
    }
    
    public static int remainderUnsigned(int n, int n2) {
        return (int)(toUnsignedLong(n) % toUnsignedLong(n2));
    }
    
    public static int highestOneBit(int n) {
        n |= n >> 1;
        n |= n >> 2;
        n |= n >> 4;
        n |= n >> 8;
        n |= n >> 16;
        return n - (n >>> 1);
    }
    
    public static int lowestOneBit(int n) {
        return n & -n;
    }
    
    public static int numberOfLeadingZeros(int n) {
        if (n == 0) {
            return 32;
        }
        int n2 = 1;
        if (n >>> 16 == 0) {
            n2 += 16;
            n <<= 16;
        }
        if (n >>> 24 == 0) {
            n2 += 8;
            n <<= 8;
        }
        if (n >>> 28 == 0) {
            n2 += 4;
            n <<= 4;
        }
        if (n >>> 30 == 0) {
            n2 += 2;
            n <<= 2;
        }
        return n2 - (n >>> 31);
    }
    
    public static int numberOfTrailingZeros(int n) {
        if (n == 0) {
            return 32;
        }
        int n2 = 31;
        int n3 = n << 16;
        if (n3 != 0) {
            n2 -= 16;
            n = n3;
        }
        int n4 = n << 8;
        if (n4 != 0) {
            n2 -= 8;
            n = n4;
        }
        int n5 = n << 4;
        if (n5 != 0) {
            n2 -= 4;
            n = n5;
        }
        int n6 = n << 2;
        if (n6 != 0) {
            n2 -= 2;
            n = n6;
        }
        return n2 - (n << 1 >>> 31);
    }
    
    public static int bitCount(int n) {
        n -= (n >>> 1 & 0x55555555);
        n = (n & 0x33333333) + (n >>> 2 & 0x33333333);
        n = (n + (n >>> 4) & 0xF0F0F0F);
        n += n >>> 8;
        n += n >>> 16;
        return n & 0x3F;
    }
    
    public static int rotateLeft(int n, int n2) {
        return n << n2 | n >>> -n2;
    }
    
    public static int rotateRight(int n, int n2) {
        return n >>> n2 | n << -n2;
    }
    
    public static int reverse(int n) {
        n = ((n & 0x55555555) << 1 | (n >>> 1 & 0x55555555));
        n = ((n & 0x33333333) << 2 | (n >>> 2 & 0x33333333));
        n = ((n & 0xF0F0F0F) << 4 | (n >>> 4 & 0xF0F0F0F));
        n = (n << 24 | (n & 0xFF00) << 8 | (n >>> 8 & 0xFF00) | n >>> 24);
        return n;
    }
    
    public static int signum(int n) {
        return n >> 31 | -n >>> 31;
    }
    
    public static int reverseBytes(int n) {
        return n >>> 24 | (n >> 8 & 0xFF00) | (n << 8 & 0xFF0000) | n << 24;
    }
    
    public static int sum(int n, int n2) {
        return n + n2;
    }
    
    public static int max(int n, int n2) {
        return Math.max(n, n2);
    }
    
    public static int min(int n, int n2) {
        return Math.min(n, n2);
    }
    
    static {
        TYPE = Class.getPrimitiveClass("int");
        digits = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z' };
        DigitTens = new char[] { '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '1', '1', '1', '1', '1', '1', '1', '1', '1', '1', '2', '2', '2', '2', '2', '2', '2', '2', '2', '2', '3', '3', '3', '3', '3', '3', '3', '3', '3', '3', '4', '4', '4', '4', '4', '4', '4', '4', '4', '4', '5', '5', '5', '5', '5', '5', '5', '5', '5', '5', '6', '6', '6', '6', '6', '6', '6', '6', '6', '6', '7', '7', '7', '7', '7', '7', '7', '7', '7', '7', '8', '8', '8', '8', '8', '8', '8', '8', '8', '8', '9', '9', '9', '9', '9', '9', '9', '9', '9', '9' };
        DigitOnes = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
        sizeTable = new int[] { 9, 99, 999, 9999, 99999, 999999, 9999999, 99999999, 999999999, Integer.MAX_VALUE };
    }
    
    private static class IntegerCache
    {
        static final int low = -128;
        static final int high;
        static final Integer[] cache;
        
        static {
            int min = 127;
            String savedProperty = VM.getSavedProperty("java.lang.Integer.IntegerCache.high");
            if (savedProperty != null) {
                try {
                    min = Math.min(Math.max(Integer.parseInt(savedProperty), 127), 2147483518);
                }
                catch (NumberFormatException ex) {}
            }
            high = min;
            cache = new Integer[Integer.IntegerCache.high + 128 + 1];
            int n = -128;
            for (int i = 0; i < Integer.IntegerCache.cache.length; ++i) {
                Integer.IntegerCache.cache[i] = new Integer(n++);
            }
            assert Integer.IntegerCache.high >= 127;
            return;
        }
    }
}
